global.vertex_format = 0;

enum N3D_BILLBOARD_TYPE {
	SPHERICAL,
	CYLINDRICAL
}

function n3d_init() {
	gpu_set_ztestenable(true);
	gpu_set_zwriteenable(true);
	
	vertex_format_begin();
	vertex_format_add_position_3d();
	vertex_format_add_normal();
	vertex_format_add_texcoord();
	vertex_format_add_color();
	global.vertex_format = vertex_format_end();
}
function n3d_create_vbuffer() {
	vbuffer = vertex_create_buffer();
	vertex_begin(vbuffer, global.vertex_format);
	return vbuffer;
}
function n3d_new_cam_fov(_x, _y, _z, _xto, _yto, _zto, _xup, _yup, _zup, _cam_width, _cam_height, _fov, _screen_aspect, _clipping_plane, _render_distance) {
	var _camera = camera_get_active();
	var _matrix = matrix_build_lookat(_x, _y, _z, _xto, _yto, _zto, _xup, _yup, _zup);
	var _proj = matrix_build_projection_perspective_fov(_fov, _screen_aspect, _clipping_plane, _render_distance);
	
	camera_set_view_size(_camera, _cam_width, _cam_height);
	window_set_size(camera_get_view_width(_camera), camera_get_view_height(_camera));
	camera_set_view_mat(_camera, _matrix);
	camera_set_proj_mat(_camera, _proj);
	
	return _camera;
}
function n3d_add_vertex_point(vbuffer, _x, _y, _z, _nx, _ny, _nz, _utex, _vtex, _color, _alpha) {
	vertex_position_3d(vbuffer, _x, _y, _z);
	vertex_normal(vbuffer, _nx, _ny, _nz);
	vertex_texcoord(vbuffer, _utex, _vtex);
	vertex_color(vbuffer, _color, _alpha);
}
function n3d_add_plane(vbuffer, _x,  _y,  _z, _width, _height, _color, _alpha) {
	n3d_add_vertex_point(vbuffer,		_x,			_y,			_z, 0, 0, 1,	0, 0,		_color,	_alpha);
	n3d_add_vertex_point(vbuffer,		_width,	_y,			_z, 0, 0, 1,	0, 1,		_color,	_alpha);
	n3d_add_vertex_point(vbuffer,		_width,	_height,	_z, 0, 0, 1,	1, 1,		_color,	_alpha);
	
	n3d_add_vertex_point(vbuffer,		_width,	_height,	_z, 0, 0, 1,	1, 1,		_color,	_alpha);
	n3d_add_vertex_point(vbuffer,		_x,			_height,	_z, 0, 0, 1,	1, 0,		_color,	_alpha);
	n3d_add_vertex_point(vbuffer,		_x,			_y,			_z, 0, 0, 1,	0, 0,		_color,	_alpha);
}
function n3d_add_rect_prism(vbuffer, _color, _shaded_color, _alpha) { 
	// left face																												y  z
	
	n3d_add_vertex_point(vbuffer,			-0.5, -0.5, 0.5,				0, 0, 1,		0, 1,		_color,			_alpha);
	n3d_add_vertex_point(vbuffer,			-0.5, 0.5, -0.5,				0, 0, 1,		1, 0,		_color,			_alpha);
	n3d_add_vertex_point(vbuffer,			-0.5, -0.5, -0.5,				0, 0, 1,		0, 0,		_color,			_alpha);

	n3d_add_vertex_point(vbuffer,			-0.5, 0.5, 0.5,					0, 0, 1,		1, 1,		_color,			_alpha);
	n3d_add_vertex_point(vbuffer,			-0.5, -0.5, 0.5,				0, 0, 1,		0, 1,		_color,			_alpha);
	n3d_add_vertex_point(vbuffer,			-0.5, 0.5, -0.5,				0, 0, 1,		1, 0,		_color,			_alpha);
	
	// right face																											y  z
	
	n3d_add_vertex_point(vbuffer,			0.5, 0.5, 0.5,					0, 0, 1,		1, 1,		_shaded_color,			_alpha);
	n3d_add_vertex_point(vbuffer,			0.5, -0.5, -0.5,				0, 0, 1,		0, 0,		_shaded_color,			_alpha);
	n3d_add_vertex_point(vbuffer,			0.5, -0.5, 0.5,					0, 0, 1,		0, 1,		_shaded_color,			_alpha);

	n3d_add_vertex_point(vbuffer,			0.5, 0.5, -0.5,					0, 0, 1,		1, 0,		_shaded_color,			_alpha);
	n3d_add_vertex_point(vbuffer,			0.5, 0.5, 0.5,					0, 0, 1,		1, 1,		_shaded_color,			_alpha);
	n3d_add_vertex_point(vbuffer,			0.5, -0.5, -0.5,				0, 0, 1,		0, 0,		_shaded_color,			_alpha);
	
	// top face																											x  y
	
	n3d_add_vertex_point(vbuffer,			-0.5, -0.5, -0.5,				0, 0, 1,		0, 0,		_color,			_alpha);
	n3d_add_vertex_point(vbuffer,			0.5, -0.5, -0.5,				0, 0, 1,		1, 0,		_color,			_alpha);
	n3d_add_vertex_point(vbuffer,			0.5, -0.5, -0.5,				0, 0, 1,		1, 1,		_color,			_alpha);

	n3d_add_vertex_point(vbuffer,			0.5, 0.5, -0.5,					0, 0, 1,		1, 1,		_color,			_alpha);
	n3d_add_vertex_point(vbuffer,			-0.5, 0.5, -0.5,				0, 0, 1,		0, 1,		_color,			_alpha);
	n3d_add_vertex_point(vbuffer,			-0.5, -0.5, -0.5,				0, 0, 1,		0, 0,		_color,			_alpha);
	
	// bottom face																										x  y
	
	n3d_add_vertex_point(vbuffer,			-0.5, -0.5, 0.5,				0, 0, 1,		0, 0,		_shaded_color,			_alpha);
	n3d_add_vertex_point(vbuffer,			0.5, -0.5, 0.5,					0, 0, 1,		1, 0,		_shaded_color,			_alpha);
	n3d_add_vertex_point(vbuffer,			0.5, 0.5, 0.5,					0, 0, 1,		1, 1,		_shaded_color,			_alpha);

	n3d_add_vertex_point(vbuffer,			0.5, 0.5, 0.5,					0, 0, 1,		1, 1,		_shaded_color,			_alpha);
	n3d_add_vertex_point(vbuffer,			-0.5, 0.5, 0.5,					0, 0, 1,		0, 1,		_shaded_color,			_alpha);
	n3d_add_vertex_point(vbuffer,			-0.5, -0.5, 0.5,				0, 0, 1,		0, 0,		_shaded_color,			_alpha);
	
	// front face																											x	z
	
	n3d_add_vertex_point(vbuffer,			-0.5, -0.5, -0.5,				0, 0, 1,		0, 0,		_color,			_alpha);
	n3d_add_vertex_point(vbuffer,			0.5, -0.5, -0.5,				0, 0, 1,		1, 0,		_color,			_alpha);
	n3d_add_vertex_point(vbuffer,			0.5, -0.5, 0.5,					0, 0, 1,		1, 1,		_color,			_alpha);
	
	n3d_add_vertex_point(vbuffer,			-0.5, -0.5, -0.5,				0, 0, 1,		0, 0,		_color,			_alpha);
	n3d_add_vertex_point(vbuffer,			-0.5, -0.5, 0.5,				0, 0, 1,		0, 1,		_color,			_alpha);
	n3d_add_vertex_point(vbuffer,			0.5, -0.5, 0.5,					0, 0, 1,		1, 1,		_color,			_alpha);
	
	// back face																											x	z
	n3d_add_vertex_point(vbuffer,			-0.5, 0.5, -0.5,				0, 0, 1,		0, 0,		_shaded_color,			_alpha);
	n3d_add_vertex_point(vbuffer,			0.5, 0.5, -0.5,					0, 0, 1,		1, 0,		_shaded_color,			_alpha);
	n3d_add_vertex_point(vbuffer,			0.5, 0.5, 0.5,					0, 0, 1,		1, 1,		_shaded_color,			_alpha);
	
	n3d_add_vertex_point(vbuffer,			-0.5, 0.5, -0.5,				0, 0, 1,		0, 0,		_shaded_color,			_alpha);
	n3d_add_vertex_point(vbuffer,			-0.5, 0.5, 0.5,					0, 0, 1,		0, 1,		_shaded_color,			_alpha);
	n3d_add_vertex_point(vbuffer,			0.5, 0.5, 0.5,					0, 0, 1,		1, 1,		_shaded_color,			_alpha);
	
}
function n3d_add_circle_plane(vbuffer, _color, _alpha) {
	// first slice
	n3d_add_vertex_point(vbuffer, 0,		-0.5,	0,			0, 0, 1,			0, 0,		c_green,		_alpha);
	n3d_add_vertex_point(vbuffer, 0.33,	-0.33,	0,			0, 0, 1,			0, 0,		c_green,		_alpha);
	n3d_add_vertex_point(vbuffer, 0,		0,		0,			0, 0, 1,			0, 0,		c_green,		_alpha);
	
	// second slice
	n3d_add_vertex_point(vbuffer, 0.5,	0,		0,			0, 0, 1,			0, 0,		c_fuchsia,		_alpha);
	n3d_add_vertex_point(vbuffer, 0.33,	-0.33,	0,			0, 0, 1,			0, 0,		c_fuchsia,		_alpha);
	n3d_add_vertex_point(vbuffer, 0,		0,		0,			0, 0, 1,			0, 0,		c_fuchsia,		_alpha);
	
	// third slice
	n3d_add_vertex_point(vbuffer, 0,		0.5,		0,			0, 0, 1,			0, 0,		c_red,		_alpha);
	n3d_add_vertex_point(vbuffer, -0.33,	0.33,		0,			0, 0, 1,			0, 0,		c_red,		_alpha);
	n3d_add_vertex_point(vbuffer, 0,		0,		0,			0, 0, 1,			0, 0,		c_red,		_alpha);
	
	// fourth slice
	n3d_add_vertex_point(vbuffer, -0.5,	0,		0,			0, 0, 1,			0, 0,		c_aqua,		_alpha);
	n3d_add_vertex_point(vbuffer, -0.33,	0.33,		0,			0, 0, 1,			0, 0,		c_aqua,		_alpha);
	n3d_add_vertex_point(vbuffer, 0,		0,		0,			0, 0, 1,			0, 0,		c_aqua,		_alpha);
	
	// fifth slice
	n3d_add_vertex_point(vbuffer, -0.33,	-0.33,	0,			0, 0, 1,			0, 0,		c_blue,		_alpha);
	n3d_add_vertex_point(vbuffer, 0,		-0.5,	0,			0, 0, 1,			0, 0,		c_blue,		_alpha);
	n3d_add_vertex_point(vbuffer, 0,		0,		0,			0, 0, 1,			0, 0,		c_blue,		_alpha);
	
	// six slice
	n3d_add_vertex_point(vbuffer, -0.33,	-0.33,	0,			0, 0, 1,			0, 0,		c_olive,		_alpha);
	n3d_add_vertex_point(vbuffer, -0.5,	0,		0,			0, 0, 1,			0, 0,		c_olive,		_alpha);
	n3d_add_vertex_point(vbuffer, 0,		0,		0,			0, 0, 1,			0, 0,		c_olive,		_alpha);
	
	// seventh slice
	n3d_add_vertex_point(vbuffer, 0.33,	0.33,		0,			0, 0, 1,			0, 0,		c_lime,		_alpha);
	n3d_add_vertex_point(vbuffer, 0,		0.5,		0,			0, 0, 1,			0, 0,		c_lime,		_alpha);
	n3d_add_vertex_point(vbuffer, 0,		0,		0,			0, 0, 1,			0, 0,		c_lime,		_alpha);
	
	// eight slice
	n3d_add_vertex_point(vbuffer, 0.33,	0.33,		0,			0, 0, 1,			0, 0,		c_purple,	_alpha);
	n3d_add_vertex_point(vbuffer, 0.5,	0,		0,			0, 0, 1,			0, 0,		c_purple,	_alpha);
	n3d_add_vertex_point(vbuffer, 0,		0,		0,			0, 0, 1,			0, 0,		c_purple,	_alpha);
}
function n3d_add_cone(vbuffer, _color, _alpha) {
	// first slice
	n3d_add_vertex_point(vbuffer, 0,			-0.5,		-0.5,				0, 0, 1,			0, 0,		_color,		_alpha);
	n3d_add_vertex_point(vbuffer, 0.33,		-0.33,		-0.5,				0, 0, 1,			0, 0,		_color,		_alpha);
	n3d_add_vertex_point(vbuffer, 0,			0,			-0.5,				0, 0, 1,			0, 0,		_color,		_alpha);
	
	// second slice
	n3d_add_vertex_point(vbuffer, 0.5,		0,			-0.5,				0, 0, 1,			0, 0,		_color,		_alpha);
	n3d_add_vertex_point(vbuffer, 0.33,		-0.33,		-0.5,				0, 0, 1,			0, 0,		_color,		_alpha);
	n3d_add_vertex_point(vbuffer, 0,			0,			-0.5,				0, 0, 1,			0, 0,		_color,		_alpha);
	
	// third slice
	n3d_add_vertex_point(vbuffer, 0,			0.5,		-0.5,				0, 0, 1,			0, 0,		_color,		_alpha);
	n3d_add_vertex_point(vbuffer, -0.33,	0.33,		-0.5,				0, 0, 1,			0, 0,		_color,		_alpha);
	n3d_add_vertex_point(vbuffer, 0,			0,			-0.5,				0, 0, 1,			0, 0,		_color,		_alpha);
	
	// fourth slice
	n3d_add_vertex_point(vbuffer, -0.5,		0,			-0.5,				0, 0, 1,			0, 0,		_color,		_alpha);
	n3d_add_vertex_point(vbuffer, -0.33,	0.33,		-0.5,				0, 0, 1,			0, 0,		_color,		_alpha);
	n3d_add_vertex_point(vbuffer, 0,			0,			-0.5,				0, 0, 1,			0, 0,		_color,		_alpha);
	
	// fifth slice
	n3d_add_vertex_point(vbuffer, -0.33,	-0.33,		-0.5,				0, 0, 1,			0, 0,		_color,		_alpha);
	n3d_add_vertex_point(vbuffer, 0,			-0.5,		-0.5,				0, 0, 1,			0, 0,		_color,		_alpha);
	n3d_add_vertex_point(vbuffer, 0,			0,			-0.5,				0, 0, 1,			0, 0,		_color,		_alpha);
	
	// six slice
	n3d_add_vertex_point(vbuffer, -0.33,	-0.33,		-0.5,				0, 0, 1,			0, 0,		_color,		_alpha);
	n3d_add_vertex_point(vbuffer, -0.5,		0,			-0.5,				0, 0, 1,			0, 0,		_color,		_alpha);
	n3d_add_vertex_point(vbuffer, 0,			0,			-0.5,				0, 0, 1,			0, 0,		_color,		_alpha);
	
	// seventh slice
	n3d_add_vertex_point(vbuffer, 0.33,		0.33,		-0.5,				0, 0, 1,			0, 0,		_color,		_alpha);
	n3d_add_vertex_point(vbuffer, 0,			0.5,		-0.5,				0, 0, 1,			0, 0,		_color,		_alpha);
	n3d_add_vertex_point(vbuffer, 0,			0,			-0.5,				0, 0, 1,			0, 0,		_color,		_alpha);
	
	// eight slice
	n3d_add_vertex_point(vbuffer, 0.33,		0.33,		-0.5,				0, 0, 1,			0, 0,		_color,		_alpha);
	n3d_add_vertex_point(vbuffer, 0.5,		0,			-0.5,				0, 0, 1,			0, 0,		_color,		_alpha);
	n3d_add_vertex_point(vbuffer, 0,			0,			-0.5,				0, 0, 1,			0, 0,		_color,		_alpha);
	
	// first slice
	n3d_add_vertex_point(vbuffer, 0,			-0.5,		-0.5,				0, 0, 1,			0, 0,		_color,		_alpha);
	n3d_add_vertex_point(vbuffer, 0.33,		-0.33,		-0.5,				0, 0, 1,			0, 0,		_color,		_alpha);
	n3d_add_vertex_point(vbuffer, 0,			0,			0.5,				0, 0, 1,			0, 0,		_color,		_alpha);
	
	// second slice
	n3d_add_vertex_point(vbuffer, 0.5,		0,			-0.5,				0, 0, 1,			0, 0,		_color,		_alpha);
	n3d_add_vertex_point(vbuffer, 0.33,		-0.33,		-0.5,				0, 0, 1,			0, 0,		_color,		_alpha);
	n3d_add_vertex_point(vbuffer, 0,			0,			0.5,				0, 0, 1,			0, 0,		_color,		_alpha);
	
	// third slice
	n3d_add_vertex_point(vbuffer, 0,			0.5,		-0.5,				0, 0, 1,			0, 0,		_color,		_alpha);
	n3d_add_vertex_point(vbuffer, -0.33,	0.33,		-0.5,				0, 0, 1,			0, 0,		_color,		_alpha);
	n3d_add_vertex_point(vbuffer, 0,			0,			0.5,				0, 0, 1,			0, 0,		_color,		_alpha);
	
	// fourth slice
	n3d_add_vertex_point(vbuffer, -0.5,		0,			-0.5,				0, 0, 1,			0, 0,		_color,		_alpha);
	n3d_add_vertex_point(vbuffer, -0.33,	0.33,		-0.5,				0, 0, 1,			0, 0,		_color,		_alpha);
	n3d_add_vertex_point(vbuffer, 0,			0,			0.5,				0, 0, 1,			0, 0,		_color,		_alpha);
	
	// fifth slice
	n3d_add_vertex_point(vbuffer, -0.33,	-0.33,		-0.5,				0, 0, 1,			0, 0,		_color,		_alpha);
	n3d_add_vertex_point(vbuffer, 0,			-0.5,		-0.5,				0, 0, 1,			0, 0,		_color,		_alpha);
	n3d_add_vertex_point(vbuffer, 0,			0,			0.5,				0, 0, 1,			0, 0,		_color,		_alpha);
	
	// six slice
	n3d_add_vertex_point(vbuffer, -0.33,	-0.33,		-0.5,				0, 0, 1,			0, 0,		_color,		_alpha);
	n3d_add_vertex_point(vbuffer, -0.5,		0,			-0.5,				0, 0, 1,			0, 0,		_color,		_alpha);
	n3d_add_vertex_point(vbuffer, 0,			0,			0.5,				0, 0, 1,			0, 0,		_color,		_alpha);
	
	// seventh slice
	n3d_add_vertex_point(vbuffer, 0.33,		0.33,		-0.5,				0, 0, 1,			0, 0,		_color,		_alpha);
	n3d_add_vertex_point(vbuffer, 0,			0.5,		-0.5,				0, 0, 1,			0, 0,		_color,		_alpha);
	n3d_add_vertex_point(vbuffer, 0,			0,			0.5,				0, 0, 1,			0, 0,		_color,		_alpha);
	
	// eight slice
	n3d_add_vertex_point(vbuffer, 0.33,		0.33,		-0.5,				0, 0, 1,			0, 0,		_color,		_alpha);
	n3d_add_vertex_point(vbuffer, 0.5,		0,			-0.5,				0, 0, 1,			0, 0,		_color,		_alpha);
	n3d_add_vertex_point(vbuffer, 0,			0,			0.5,				0, 0, 1,			0, 0,		_color,		_alpha);
}
function n3d_draw_vbuffer_init(vbuffer) {
	vertex_end(vbuffer);
}
function n3d_draw_vbuffer(vbuffer, texture, primitive_type) {
	if (texture != -1) {
		vertex_submit(vbuffer, primitive_type, sprite_get_texture(texture, 0));
	} else 	vertex_submit(vbuffer, primitive_type, texture);
}
function n3d_destroy() {
	gpu_set_zwriteenable(false);
	gpu_set_ztestenable(false);
}
function n3d_draw_3d_sprite(_spr_index, _image_index, _x, _y, _z, drawtype) {
	switch(drawtype) {
		case N3D_BILLBOARD_TYPE.SPHERICAL:
			shader_set(sh_n3d_sph_billboard);
			matrix_set(matrix_world, matrix_build(x, y, z, 0, 0, 0, 1, 1, 1));
			draw_sprite(_spr_index, _image_index, 0, 0);
			matrix_set(matrix_world, matrix_build_identity());
			shader_reset();
		break;
		case N3D_BILLBOARD_TYPE.CYLINDRICAL:
			shader_set(sh_n3d_cyl_billboard);
			matrix_set(matrix_world, matrix_build(x, y, z, 0, 0, 0, 1, 1, 1));
			draw_sprite(_spr_index, _image_index, 0, 0);
			matrix_set(matrix_world, matrix_build_identity());
			shader_reset();
		break;
	}
}